#include <gl/glu.h>
#include <SDL/SDL.h>
#include <windows.h>

#include "Cube.hpp"
#include "Matrice.hpp"

#define ECRAN_LARGEUR 800
#define ECRAN_HAUTEUR 600
#define FPS 24

LRESULT CALLBACK WindowProc ( HWND , UINT , WPARAM , LPARAM ) ;
void EnableOpenGL ( HWND hwnd , HDC* , HGLRC* ) ;
void DisableOpenGL ( HWND , HDC , HGLRC ) ;

int WINAPI WinMain ( HINSTANCE hInstance , HINSTANCE hPrevInstance , LPSTR lpCmdLine , int nCmdShow )
{
    WNDCLASSEX wcex ;
    HWND hwnd ;
    HDC hDC ;
    HGLRC hRC ;
    MSG msg ;
    BOOL bQuit = FALSE ;
    float theta = 0.0f ;

    Cube CubeTest ( 5 , 5 , 5 , 50 , 50 , 255 ) ;
    Matrice MatriceTest ( 10 , 10 , 2 , 2 , ".\\Textures\\Herbe.bmp" , 2 , 2 ) ;


    int TimerFPS = 0 ;

    /* register window class */
    wcex.cbSize = sizeof ( WNDCLASSEX ) ;
    wcex.style = CS_OWNDC ;
    wcex.lpfnWndProc = WindowProc ;
    wcex.cbClsExtra = 0 ;
    wcex.cbWndExtra = 0 ;
    wcex.hInstance = hInstance ;
    wcex.hIcon = LoadIcon ( NULL , IDI_APPLICATION ) ;
    wcex.hCursor = LoadCursor ( NULL , IDC_ARROW ) ;
    wcex.hbrBackground = ( HBRUSH ) GetStockObject ( BLACK_BRUSH ) ;
    wcex.lpszMenuName = NULL ;
    wcex.lpszClassName = "GLSample" ;
    wcex.hIconSm = LoadIcon ( NULL , IDI_APPLICATION ) ;

    if ( !RegisterClassEx ( &wcex ) )
        return 0 ;

    // Cration de la fentre principale
    hwnd = CreateWindowEx( 0 , "GLSample" , "Slimes Fever" , WS_OVERLAPPEDWINDOW , CW_USEDEFAULT , CW_USEDEFAULT , ECRAN_LARGEUR , ECRAN_HAUTEUR , NULL , NULL , hInstance , NULL ) ;

    ShowWindow ( hwnd , nCmdShow ) ;

    // Initialisation de l'opengl
    EnableOpenGL ( hwnd , &hDC , &hRC ) ;
    glEnable ( GL_DEPTH_TEST ) ;
    glEnable ( GL_TEXTURE_2D ) ;
    glMatrixMode ( GL_MODELVIEW ) ;
    glLoadIdentity ( ) ;
    gluPerspective ( 45.0f , ( ECRAN_LARGEUR / ECRAN_HAUTEUR ) , 1.0f , 1000.0f ) ;
    glHint ( GL_PERSPECTIVE_CORRECTION_HINT , GL_NICEST ) ;

    // Place de la camra vers la cible
    gluLookAt ( 0 , 10 , 50 , 0 , 0 , 0 , 0 , 1 , 0 ) ;

    // Placement du cube
    CubeTest.SetPos ( 0 , 10 , 0 ) ;

    // Matrices
    MatriceTest.Aleatoire ( -1 , 1 ) ;
    MatriceTest.SetPosition ( -10 , 5 , -10 ) ;

    // Boucle principale
    while ( !bQuit )
    {
        // Vrification des messages
        if ( PeekMessage ( &msg , NULL , 0 , 0 , PM_REMOVE ) )
        {
            // Prise en charge des messages
            if ( msg.message == WM_QUIT )
            {
                bQuit = TRUE ;
            }
            else
            {
                TranslateMessage ( &msg ) ;
                DispatchMessage ( &msg ) ;
            }
        }
        else
        {
            // Animatin opengl
            glClearColor ( 0.0f , 0.0f , 0.0f , 0.0f ) ;
            glClear ( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT ) ;

            glPushMatrix ( ) ;
            glRotatef ( theta , 0.0f , 1.0f , 0.0f ) ;

            // Test
            CubeTest.Afficher ( ) ;
            MatriceTest.Afficher ( ) ;

            glPopMatrix ( ) ;

            SwapBuffers ( hDC ) ;

            theta += 1.0f ;
        }

        // Boucle de contrle du FPS
        if ( SDL_GetTicks ( ) - TimerFPS < ( 1000 / FPS ) )
            SDL_Delay ( ( ( 1000 ) / FPS )  - ( SDL_GetTicks ( ) - TimerFPS ) ) ;
        TimerFPS = SDL_GetTicks ( ) ;
    }

    //Fermeture d'opengl
    DisableOpenGL(hwnd, hDC, hRC);

    // Fermeture de la fentre
    DestroyWindow(hwnd);

    return msg.wParam ;
}

LRESULT CALLBACK WindowProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    switch ( uMsg )
    {
        case WM_CLOSE :
            PostQuitMessage ( 0 ) ;
        break ;

        case WM_DESTROY :
            return 0 ;

        case WM_KEYDOWN :
        {
            switch ( wParam )
            {
                case VK_ESCAPE :
                    PostQuitMessage ( 0 ) ;
                break ;
            }
        }
        break ;

        default :
            return DefWindowProc ( hwnd , uMsg , wParam , lParam ) ;
    }

    return 0 ;
}

void EnableOpenGL ( HWND hwnd , HDC* hDC , HGLRC* hRC )
{
    PIXELFORMATDESCRIPTOR pfd ;

    int iFormat ;

    /* get the device context (DC) */
    *hDC = GetDC ( hwnd ) ;

    /* set the pixel format for the DC */
    ZeroMemory ( &pfd , sizeof ( pfd ) ) ;

    pfd.nSize = sizeof ( pfd ) ;
    pfd.nVersion = 1 ;
    pfd.dwFlags = PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL | PFD_DOUBLEBUFFER ;
    pfd.iPixelType = PFD_TYPE_RGBA ;
    pfd.cColorBits = 24 ;
    pfd.cDepthBits = 16 ;
    pfd.iLayerType = PFD_MAIN_PLANE ;

    iFormat = ChoosePixelFormat ( *hDC , &pfd ) ;

    SetPixelFormat ( *hDC , iFormat , &pfd ) ;

    /* create and enable the render context (RC) */
    *hRC = wglCreateContext ( *hDC ) ;

    wglMakeCurrent ( *hDC , *hRC ) ;
}

void DisableOpenGL ( HWND hwnd , HDC hDC , HGLRC hRC )
{
    wglMakeCurrent ( NULL, NULL ) ;
    wglDeleteContext ( hRC ) ;
    ReleaseDC ( hwnd , hDC ) ;
}
